#! /bin/sh


##############################################################################
#                     FONCTIONS DE STOCKAGE                                  #
##############################################################################


hash_add()
{
    if [ "$#" -lt "3" ] ; then
	echo '(hash_add) Usage: hash_add name_hash key value' 1>&2
	return 1
    fi
    name_hash=$1
    key_hash=$2
    value_hash=$3
    tmp=`eval echo "$\`eval echo ${name_hash}_Keys\` \'$key_hash\'"`
    eval ${name_hash}_Keys=\$tmp
    eval ${name_hash}_${key_hash}=\$value_hash
    return 0
}

hash_replace()
{
    if [ "$#" -lt "3" ] ; then
	echo '(hash_replace) Usage: hash_replace name_hash key new_value' 1>&2
	return 1
    fi
    name_hash=$1
    key_hash=$2
    value_hash=$3
    if [ -z "`eval echo $\`eval echo ${name_hash}_${key_hash}\``" ] ; then
	echo "(hash_replace) Warning: Value uninitialized with key '${key_hash}'" 1>&2
    fi
    eval ${name_hash}_${key_hash}=\$value_hash
    return 0
}


##############################################################################
#                     FONCTIONS D'EFFACEMENT                                 #
##############################################################################


hash_delete()
{
    name_hash=$1
    key_hash_erase=$2
    if [ -z "`eval echo $\`eval echo ${name_hash}_${key_hash_erase}\``" ]
	then
	echo "(hash_delete) No value associated with key $key_hash_erase" 1>&2
	return 1
    fi
    tmp=`eval echo "$\`eval echo ${name_hash}_Keys\`" | sed "s/'$key_hash_erase'//"`
    eval ${name_hash}_Keys=\$tmp
    eval unset ${name_hash}_${key_hash_erase}
    return 0
}

hash_clean()
{
    name_hash=$1
    for key in `hash_keys $name_hash` 
      do
      eval unset ${name_hash}_${key}
    done
    eval unset ${name_hash}_Keys
    return 0
}


##############################################################################
#                     FONCTIONS DE CONTROLE                                  #
##############################################################################


hash_keys()
{
    name_hash=$1
    if [ "$#" -lt "1" ] ; then
	echo '(hash_keys) Usage: hash_keys name_hash' 1>&2
	return 1
    fi
    if [ -z "`eval echo $\`eval echo ${name_hash}_Keys\``" ] ; then
	echo "(hash_keys) uninitialized hash table '${name_hash}'" 1>&2
	return 1
    fi
    eval echo $`eval echo ${name_hash}_Keys` | sed "s/'\([a-zA-Z0-9_]*\)'/\\1/g" 
    return 0
}

hash_sort_keys()
{
    name_hash=$1
    opt_sort=$2
    eval echo $`eval echo ${name_hash}_Keys` | sed "s/'\([a-zA-Z0-9_]*\)'/\\1/g" | tr " " "\n" | eval sort $opt_sort
    return 0
}

hash_values()
{
    name_hash=$1
    for key in `hash_keys $name_hash`
      do
      hash_aff_value $name_hash $key
    done
    return 0
}


##############################################################################
#                     FONCTIONS D'AFFICHAGE                                  #
##############################################################################


hash_aff_value()
{
    name_hash=$1
    key_hash=$2
    if [ -z "`eval echo $\`eval echo ${name_hash}_${key_hash}\``" ] ; then
	echo "(hash_aff_value) Value uninitialized with key '${key_hash}'" 1>&2
	return 1
    fi
    eval echo "$`eval echo ${name_hash}_${key_hash}`"
    return 0
}


hash_print()
{
    name_hash=$1
    for key in `hash_sort_keys $name_hash`
      do
      echo $key "=>" `hash_aff_value $name_hash $key`
    done
}

##############################################################################
#                     FONCTIONS EXOTIQUE                                     #
##############################################################################


hash_swap_all()
{
    if [ "$#" -lt "1" ] ; then
	echo '(hash_swap_all) Usage: hash_swap_all name_hash' 1>&2
	return 1
    fi
    name_hash=$1
    all_values=`hash_values $name_hash`
    all_values_tmp=`echo $all_values | sed "s/\([A-Za-z0-9_]*\)/'\\1'/g"`
    for key in `hash_keys $name_hash` ; do
	hash_replace $name_hash `hash_aff_value $name_hash $key 2> /dev/null` $key 2> /dev/null
	echo $all_values_tmp | grep "'$key'" > /dev/null 2>&1 
	if [ "$?" -eq "1" ] ; then
	    hash_delete $name_hash $key
	fi
    done
    eval ${name_hash}_Keys=\$all_values_tmp
    unset all_values
    unset all_values_tmp
    return 0
}

hash_swap()
{
    if [ "$#" -lt "2" ] ; then
	echo '(hash_swap_all) Usage: hash_swap name_hash key_hash' 1>&2
	return 1
    fi
    name_hash=$1
    key_hash=$2
    value=`hash_aff_value $name_hash $key_hash 2> /dev/null`
    if [ -z "$value" ] ; then
	echo "(hash_swap) Value uninitialized with key '${key_hash}'" 1>&2
	return 1
    fi
    hash_delete $name_hash $key_hash
    hash_add $name_hash $value $key_hash
    unset $value
}

hash_dup()
{
    name_hash=$1
    dup_hash=$2
    all_values_tmp=`echo $all_values | sed "s/\([A-Za-z0-9_]*\)/'\\1'/g"`
    for key in `hash_keys $name_hash` ; do
	hash_add $dup_hash $key `hash_aff_value $name_hash $key`
    done
    eval ${dup_hash}_Keys=\$all_values_tmp
}
